<?php
/**
 * Add custom functionality to use Apple Pay via the Payment Request Button.
 *
 * @link https://stripe.com/docs/stripe-js/elements/payment-request-button#verifying-your-domain-with-apple-pay
 *
 * @since 3.4.0
 */

namespace SimplePay\Pro\Admin\Apple_Pay;

use SimplePay\Core\Payments\Stripe_API;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Return information associated with the name/location of the domain verification file.
 *
 * @since 3.4.0
 *
 * @return array Domain verification file information.
 */
function get_fileinfo() {
	$path = untrailingslashit( $_SERVER['DOCUMENT_ROOT'] );
	$dir  = '.well-known';
	$file = 'apple-developer-merchantid-domain-association';

	return array(
		'path'     => $path,
		'dir'      => $dir,
		'file'     => $file,
		'fullpath' => $path . '/' . $dir . '/' . $file,
	);
}

/**
 * Determine is the current domain has been verified previously.
 * Looks for the `apple_pay_domain_verification` key in the `simpay_settings_keys` option.
 *
 * Verification is valid if the saved value matches the current domain.
 *
 * @since 3.4.0
 *
 * @return bool If the current domain has been previously verified.
 */
function is_current_domain_verified() {
	$settings = get_option( 'simpay_settings_keys' );

	if ( ! isset( $settings['apple_pay_domain_verification'] ) ) {
		return false;
	}

	if ( $_SERVER['HTTP_HOST'] !== $settings['apple_pay_domain_verification'] ) {
		return false;
	}

	return true;
}

/**
 * Determine if the domain verification file already exists.
 *
 * @since 3.4.0
 *
 * @return bool If the domain verification file exists.
 */
function has_verification_file() {
	$fileinfo = get_fileinfo();

	if ( ! @file_exists( $fileinfo['fullpath'] ) ) {
		return false;
	}

	return true;
}

/**
 * Attempt to create a directory in the server root and copy the domain verification file.
 *
 * @since 3.4.0
 *
 * @throws \Exception If the directory or file cannot be created.
 */
function create_directory_and_move_file() {
	$fileinfo = get_fileinfo();

	// Create directory if it does not exist.
	if ( ! file_exists( trailingslashit( $fileinfo['path'] ) . $fileinfo['dir'] ) ) {
		if ( ! @mkdir( trailingslashit( $fileinfo['path'] ) . $fileinfo['dir'], 0755 ) ) { // @codingStandardsIgnoreLine
			throw new \Exception( __( 'Unable to create domain association folder in domain root.', 'simple-pay' ) );
		}
	}

	// Move file if needed.
	if ( ! has_verification_file() ) {
		if ( ! @copy( trailingslashit( SIMPLE_PAY_DIR ) . $fileinfo['file'], $fileinfo['fullpath'] ) ) { // @codingStandardsIgnoreLine
			throw new \Exception( __( 'Unable to copy domain association file to domain .well-known directory.', 'simple-pay' ) );
		}
	}
}

/**
 * Verify domain with Stripe.
 *
 * Must be a valid domain, and a domain that is publicly accessible.
 *
 * @since 3.4.0
 *
 * @throws \Exception If the API request fails.
 */
function verify_domain_with_stripe() {
	$create = Stripe_API::request(
		'ApplePayDomain',
		'create',
		array(
			'domain_name' => $_SERVER['HTTP_HOST'],
		)
	);

	if ( ! $create ) {
		throw new \Exception( __( 'Unable to automatically add domain to Apple Pay settings in Stripe.', 'simple-pay' ) );
	}

	return $create;
}

/**
 * Determine if Apple Pay domain verification is needed.
 *
 * 1. Stripe keys are set.
 * 2. The form has a "Payment Request Butotn" field added.
 * 3. There is no previously saved verification process.
 *
 * @since 3.4.0
 */
function maybe_add_apple_pay_domain_verification() {
	// No keys exist, or in test mode, we can't do anything.
	if ( ! simpay_check_keys_exist() || simpay_is_test_mode() ) {
		return;
	}

	$fields  = isset( $_POST['_simpay_custom_field'] ) ? $_POST['_simpay_custom_field'] : array();
	$has_prb = false;

	foreach ( $fields as $type => $field ) {
		if ( 'payment_request_button' === $type ) {
			$has_prb = true;
			break;
		}
	}

	// No Payment Request Button field exists.
	if ( ! $has_prb ) {
		return;
	}

	// Correct domain already verified and file exists.
	if ( is_current_domain_verified() && has_verification_file() ) {
		return;
	}

	$settings = get_option( 'simpay_settings_keys' );

	try {
		// Create directory and move file if needed.
		create_directory_and_move_file();

		// Verify domain with Stripe
		$verification = verify_domain_with_stripe();

		// Set domain verification to current domain name.
		$settings['apple_pay_domain_verification'] = $verification['domain_name'];
	} catch ( \Exception $e ) {
		// Set error if something went wrong.
		$settings['apple_pay_domain_verification'] = $e->getMessage();

		// Show error if necessary.
		show_apple_pay_domain_verification_error();
	}

	update_option( 'simpay_settings_keys', $settings );
}
add_action( 'simpay_save_form_settings', __NAMESPACE__ . '\\maybe_add_apple_pay_domain_verification' );

/**
 * If an error was caught trying to generate the domain verification, show it.
 *
 * @since 3.4.0
 */
function show_apple_pay_domain_verification_error() {
	$settings = get_option( 'simpay_settings_keys' );
	$error    = (
		isset( $settings['apple_pay_domain_verification'] ) &&
		$settings['apple_pay_domain_verification'] !== $_SERVER['HTTP_HOST']
	);
?>

<div class="error">
	<p>
		<strong><?php echo wp_kses_post( __( 'Apple Pay domain verification error', 'simple-pay' ) ); ?></strong><?php echo $error ? wp_kses_post( ': ' . $settings['apple_pay_domain_verification'] ) : null; ?>
	</p>
	<p>
		<a href="https://stripe.com/docs/stripe-js/elements/payment-request-button#verifying-your-domain-with-apple-pay">
			<?php echo esc_html__( 'Manually verify your domain', 'simple-pay' ); ?>
		</a>
	</p>
</div>

<?php
}
