<?php
/**
 * reCAPTCHA functionality.
 *
 * @since 3.5.0
 */

namespace SimplePay\Pro\reCAPTCHA;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Determine if keys are entered.
 *
 * @since 3.5.0
 *
 * @return bool
 */
function has_keys() {
	return get_key( 'site' ) && get_key( 'secret' );
}

/**
 * Retrieve a site key.
 *
 * @since 3.5.0
 *
 * @param string $key Type of key to retrieve. `site` or `secret`.
 * @return bool|string
 */
function get_key( $key ) {
	$settings = get_option( 'simpay_settings_general' );
	$key      = isset( $settings['recaptcha'][ $key ] ) ? $settings['recaptcha'][ $key ] : false;

	if ( ! $key || '' === $key ) {
		return false;
	}

	return $key;
}

/**
 * Generate custom HTML to output under reCAPTCHA title.
 *
 * @since 3.5.0
 */
function admin_setting_description() {
	ob_start();
?>

	<p><?php esc_html_e( 'reCAPTCHA can help automatically protect your custom payment forms from spam and fraud.', 'simple-pay' ); ?></p>

	<br />

	<p>
	<?php
	echo wp_kses_post(
		sprintf(
			/* translators: %1$s Opening anchor tag, do not translate. %2$s Closing anchor tag, do not translate. */
			__( 'To enable reCAPTCHA %1$sregister your site with Google%2$s with reCAPTCHA v3 to retrieve the necessary credentials.', 'simple-pay' ),
			'<a href="https://www.google.com/recaptcha/admin/create" target="_blank" rel="noopener noreferrer">',
			'</a>'
		)
	);
	?>
	</p>

	<br />

	<p>
	<?php
	echo wp_kses_post(
		sprintf( 
			/* translators: %1$s Opening anchor tag, do not translate. %2$s Closing anchor tag, do not translate. */
			__( 'For more information view our %1$shelp docs for reCAPTCHA%2$s.', 'simple-pay' ),
			'<a href="' . simpay_docs_link( '', 'recaptcha', 'global-settings', true ) . '" target="_blank" rel="noopener noreferrer">',
			'</a>'
		)
	);
	?>
	</p>

<?php
	return ob_get_clean();
}

/**
 * Add "reCaptcha" section to General tab.
 *
 * @since 3.5.0
 *
 * @param array $sections Settings sections.
 * @return array
 */
function add_recaptcha_settings_section( $sections ) {
	$sections['recaptcha'] = array(
		'title' => __( 'reCAPTCHA', 'simple-pay' ),
	);

	return $sections;
}
add_filter( 'simpay_add_settings_general_sections', __NAMESPACE__ . '\\add_recaptcha_settings_section' );

/**
 * Add "reCAPTCHA" fields to General tab.
 *
 * @since 3.5.0
 *
 * @param array $fields Settings fields.
 * @return array
 */
function add_recaptcha_settings_fields( $fields ) {
	$id      = 'settings';
	$group   = 'general';
	$section = 'recaptcha';

	$input_args = array(
		'type'        => 'standard',
		'subtype'     => 'password',
		'default'     => '',
		'class'       => array(
			'regular-text',
		),
	);

	$fields[ $section ]['setup'] = array(
		'title'       => esc_html__( 'Setup', 'simple-pay' ),
		'type'        => 'custom-html',
		'html'        => admin_setting_description(),
		'name'        => 'simpay_' . $id . '_' . $group . '[' . $section . '][setup]',
		'id'          => 'simpay-' . $id . '-' . $group . '-' . $section . '-setup',
	);

	$fields[ $section ]['site'] = wp_parse_args(
		array(
			'subtype' => 'text',
			'title'   => esc_html__( 'Site Key', 'simple-pay' ),
			'name'    => 'simpay_' . $id . '_' . $group . '[recaptcha][site]',
			'id'      => 'simpay-' . $id . '-' . $group . '-recaptcha-site',
			'value'   => get_key( 'site' ),
		),
		$input_args
	);

	$fields[ $section ]['secret'] = wp_parse_args(
		array(
			'title' => esc_html__( 'Secret Key', 'simple-pay' ),
			'name'  => 'simpay_' . $id . '_' . $group . '[recaptcha][secret]',
			'id'    => 'simpay-' . $id . '-' . $group . '-recaptcha-secret',
			'value' => get_key( 'secret' ),
		),
		$input_args
	);

	return $fields;
}
add_filter( 'simpay_add_settings_general_fields', __NAMESPACE__ . '\\add_recaptcha_settings_fields' );

/**
 * Enqueue scripts necessary for generating a reCAPTCHA token.
 *
 * @since 3.5.0
 *
 * @param int    $form_id Current Form ID.
 * @param object $form Current form.
 */
function add_script( $form_id, $form ) {
	// Does not integrate with Stripe Checkout.
	if ( 'stripe_checkout' === $form->get_form_display_type() ) {
		return;
	}

	// No keys are entered.
	if ( ! has_keys() ) {
		return;
	}

	$url = add_query_arg(
		array(
			'render' => get_key( 'site' ),
		),
		'https://www.google.com/recaptcha/api.js?render=reCAPTCHA_site_key'
	);

	wp_enqueue_script( 'google-recaptcha', esc_url( $url ), array(), 'v3', true );

	wp_localize_script(
		'google-recaptcha',
		'simpayGoogleRecaptcha',
		array(
			'siteKey' => get_key( 'site' ),
		)
	);

	wp_enqueue_script(
		'simpay-notices',
		SIMPLE_PAY_ASSETS . 'js/simpay-pro-recaptcha.min.js',
		array(
			'wp-util',
			'underscore',
			'google-recaptcha',
			'simpay-public-pro',
		),
		SIMPLE_PAY_VERSION,
		true
	);
}
add_action( 'simpay_form_before_form_bottom', __NAMESPACE__ . '\\add_script', 10, 2 );

/**
 * Validate a reCAPTCHA token on form submission.
 *
 * @since 3.5.0
 */
function validate_recaptcha() {
	global $simpay_form;

	// Does not integrate with Stripe Checkout.
	if ( 'stripe_checkout' === $simpay_form->get_form_display_type() ) {
		return;
	}

	// No keys are entered.
	if ( ! has_keys() ) {
		return;
	}

	$token = isset( $_POST['simpay_recaptcha_response'] ) ? esc_attr( $_POST['simpay_recaptcha_response'] ) : false;

	// A token couldn't be generated, let it through.
	if ( ! $token ) {
		return;
	}

	$secret = get_key( 'secret' );

	$request = wp_remote_post(
		'https://www.google.com/recaptcha/api/siteverify',
		array(
			'body' => array(
				'secret'   => $secret,
				'response' => $token,
			),
		)
	);

	$no_verification = esc_html__( 'Unable to verify Google reCAPTCHA response.', 'simple-pay' );

	// Request fails.
	if ( is_wp_error( $request ) ) {
		wp_die( $no_verification );
	}

	$response = json_decode( wp_remote_retrieve_body( $request ), true );

	// No score available.
	if ( ! isset( $response['score'] ) ) {
		$no_verification .= ' ' . esc_html__( 'No score threshold available in response.', 'simple-pay' );
		wp_die( $no_verification );
	}

	// Actions do not match.
	if ( isset( $response['action'] ) && 'simple_pay_form_' . $simpay_form->id !== $response['action'] ) {
		$no_verification .= ' ' . esc_html__( 'Response action does not match request action.', 'simple-pay' );
		wp_die( $no_verification );
	}

	/**
	 * Filter the minimum score allowed for a reCAPTCHA response to allow form submission.
	 *
	 * @since 3.5.0
	 *
	 * @param string $minimum_score Minumum score.
	 */
	$minimum_score = apply_filters( 'simpay_recpatcha_minimum_score', '0.5' );

	if ( floatval( $response['score'] ) < floatval( $minimum_score ) ) {
		$recaptcha_fail_message = esc_html__( 'Unable to verify the validity of this payment request. If you believe this to be an error please contact the website administrator.', 'simple-pay' );

		/**
		 * Filter the message shown to a "user" when a reCAPTCHA score is too low.
		 *
		 * @since 3.5.0
		 *
		 * @param string $recaptcha_fail_message
		 */
		$recaptcha_fail_message = apply_filters( 'simpay_recaptcha_fail_message', $recaptcha_fail_message );

		return wp_die( $recaptcha_fail_message );
	}
}
add_action( 'simpay_pre_process_form', __NAMESPACE__ . '\\validate_recaptcha' );
