<?php
/**
 * Invoice payment successful.
 *
 * @since 3.5.0
 */

namespace SimplePay\Pro\Webhooks;

use SimplePay\Core\Payments\Stripe_API;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Invoice_Payment_Succeeded class.
 *
 * @since 3.5.0
 */
class Webhook_Invoice_Payment_Succeeded extends Webhook_Base implements Webhook_Interface {

	/**
	 * Handle the Webhook's data.
	 *
	 * @todo Ref subscription in first line item of invoice instead? Don't think it matters though.
	 * @see https://stripe.com/docs/recipes/installment-plan
	 *
	 * @since 3.5.0
	 */
	public function handle() {
		$invoice      = $this->event->data->object;
		$subscription = Stripe_API::request( 'Subscription', 'retrieve', $invoice->subscription );

		// @todo API call should already be throwing this exception, but it returns `false` on failure.
		if ( ! $subscription ) {
			throw new \Exception( esc_html__( 'Subscription not found.', 'simple-pay' ) );
		}

		/**
		 * Allow additional actions to be performed inside the `invoice.payment_succeeded` event processing.
		 *
		 * @since 3.5.0
		 *
		 * @param object $this->event Stripe Event object.
		 * @param object $invoice Stripe Invoice object.
		 * @param object $subscription Stripe Subscription object.
		 */
		do_action( 'simpay_webhook_invoice_payment_succeeded', $this->event, $invoice, $subscription );

		// No max charge is set, so do nothing.
		if ( ! isset( $subscription->metadata['simpay_charge_max'] ) ) {
			return;
		}

		$max_charges  = $subscription->metadata['simpay_charge_max'];
		$charge_count = $subscription->metadata['simpay_charge_count'];

		$charge_count++;

		// Update the total count metadata
		$subscription->metadata['simpay_charge_count'] = absint( $charge_count );
		$subscription->save();

		/**
		 * Allow additional actions to be performed before subscription metadata is updated.
		 *
		 * Since 3.5.0 this now actually happens *after* the subscription is updated in Stripe.
		 *
		 * @since 3.5.0
		 *
		 * @param object $this->event Stripe Event object.
		 * @param object $invoice Stripe Invoice object.
		 * @param object $subscription Stripe Subscription object.
		 */
		do_action( 'simpay_webhook_after_installment_increase', $this->event, $invoice, $subscription );

		// Cancel subscription if the new charge count equals (or is somehow greater) than the max charges.
		if ( $charge_count >= $max_charges ) {
			$subscription->cancel();

			/**
			 * Allow additional actions to be performed after a subscription is cancelled.
			 *
			 * @since 3.5.0
			 *
			 * @param object $this->event Stripe Event object.
			 * @param object $invoice Stripe Invoice object.
			 * @param object $subscription Stripe Subscription object.
			 */
			do_action( 'simpay_webhook_after_subscription_cancel', $this->event, $invoice, $subscription );
		}
	}
}
